%% Licensed Materials - Property of IBM
%% Restricted Materials of IBM. Reproduction, modification and redistribution are prohibited.
%% IBM Rational Rhapsody Design Manager
%%  Copyright IBM Corporation 2011 
%% U.S. Government Users Restricted Rights: Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp.

% ================================================================
% --- Load Simulink model and serialize it into index files:
% ================================================================
function rdm_serializer(modelFile, workingFolder, javaFolder)

% PRINT INPUT PARAMETERS
fprintf('\n===========================\n');
fprintf('Input Parameters:\n');
fprintf('===========================\n');
fprintf('Simulink model file=%s\n', modelFile);
fprintf('working folder=%s\n', workingFolder);
fprintf('java folder=%s\n', javaFolder);
fprintf('===========================\n\n');

% EXECUTE IMPORT
sys = load_system(modelFile);
if isobject(sys) == 0

    sysName = get_param(sys,'name');
    
    mkdir(workingFolder);
    pubDir = strcat(workingFolder, '/',sysName);
    mkdir(pubDir);
    
    % Load java libraries:
    files = dir(javaFolder);
    nfiles = size(files);
    
    for x = 1:nfiles
        filename = files(x).name;
        if strcmp(filename,'.') == 0 && strcmp(filename,'..') == 0
            javaaddpath(strcat(javaFolder,'/', filename));
        end
    end

    % Alocate java objects:
    modelParser = javaObject('com.ibm.rational.rdm.simulinkintegration.publish.ModelSerializer', pubDir);
    if isjava(modelParser) == 0 
        fprintf('ERROR: java object (pub) allocation failed\n');
        exit;
    end
    
    helper = javaObject('com.ibm.rational.rdm.simulinkintegration.publish.MscriptHelper');
    if isjava(helper) == 0 
        fprintf('ERROR: java object (helper) allocation failed\n');
        exit;
    end
    
    modelParser.setModelFile(modelFile);
    modelParser.createModel(sysName);
    
    getSystemData(sys, modelParser, '', helper, pubDir);
    
    % Serialize:
    modelParser.Serialize();
   
    % Finalize:
    close_system(modelFile, 0);
    
    exit;
end

% ==================
% getSystemData
% ==================
function getSystemData(sys, modelParser, parentSysPath, helper, publishDir)
    
    internalSysName = get_param(sys,'name');
    sysName = regexprep(internalSysName, '\n', ' ');
    sysDescript = get_param(sys,'Description');
    
    sysUriName = char(helper.legalizeMdlElemName(sysName));
    
    subSystem = 0;
    
     % add system object to publisher:
     if ~isempty(parentSysPath)
         modelParser.addSystem(sysName, sysUriName, sysDescript, 0);
         subSystem = 1;
     else
         modelParser.addSystem(sysName, sysUriName, sysDescript, 1);
     end
    
    legalParentSysPath = regexprep(parentSysPath, '\n', ' ');

    sysPath = sysName;
    if isempty(legalParentSysPath) == 0
        sysPath = strcat(legalParentSysPath,'/',sysName);
    end
            
    % Print diagram:
    % ======================
    isStateFlow = '';
    try
        isStateFlow = get_param(sysPath, 'MaskType');
    catch
        fprintf('No MaskType found for %s\n', sysPath);
    end
    
    if strcmp(isStateFlow, 'Stateflow') == 1
        printChartDiagram(sys, sysPath, publishDir, helper);
    else
        printSystemDiagram(sys, publishDir, helper);
    end
    
    % Get system lines:
    % ======================
    SysLines = get_param (sys,'Lines');
    numOfLines = size(SysLines);
    
    for z = 1:numOfLines
        lineName = SysLines(z).Name;
        srcBlock = get_param(SysLines(z).SrcBlock, 'Name');
        legalSrcBlock = char(helper.legalizeMdlElemName(srcBlock));
        destBlock = get_param(SysLines(z).DstBlock, 'Name');
        legalDestBlock = char(helper.legalizeMdlElemName(destBlock));
        
        fprintf('adding signal [%s]. SrcBlock:%s, DestBlock:%s\n', lineName, legalSrcBlock, legalDestBlock);
        modelParser.addSignal(lineName, legalSrcBlock, legalDestBlock);
    end
    
    % Get system blocks:
    % ======================
    blocks = get_param (sys,'Blocks');
    numOfBlocks = size(blocks);

    for x = 1:numOfBlocks
        
        tempBlockName = blocks{x};
        internalBlockName = regexprep(tempBlockName, '/', '//');
        legalBlockName = regexprep(internalBlockName, '\n', ' ');
        tempBlockPath = strcat(sysPath,'/',legalBlockName);
        internalBlockPath = horzcat(sysPath, '/',legalBlockName);
        blockPath = regexprep(tempBlockPath, '\n', ' ');

        if subSystem == 1
            legalSysPath = char(helper.legalizeMdlElemName(sysPath));
            legalBlockName2 = char(helper.legalizeMdlElemName(legalBlockName));
            uriName = strcat(legalSysPath, '_subsys_', legalBlockName2);
        else
            uriName = char(helper.legalizeMdlElemName(blockPath));
        end

        
        blockPosition = get_param (internalBlockPath,'Position');
        blockType = get_param (internalBlockPath,'BlockType');
        blockDesc = get_param (internalBlockPath,'Description');
        
        dblArray = javaArray ('java.lang.Double', 4);

        dblArray(1) = java.lang.Double(blockPosition(1));
        dblArray(2) = java.lang.Double(blockPosition(2));
        dblArray(3) = java.lang.Double(blockPosition(3));
        dblArray(4) = java.lang.Double(blockPosition(4));
        
        modelParser.addBlock(sysUriName, legalBlockName, uriName, blockType, blockDesc, dblArray);
        
        ObjParamsStruct = get_param(internalBlockPath, 'DialogParameters');
        numOfParams = 0;
        try
            ObjParams = struct2cell(ObjParamsStruct);
            InternalNames = fieldnames(ObjParamsStruct);
            numOfParams = size(ObjParams);
        catch
            fprintf('ERROR: struct2cell() failure\n');
        end
        
        getProps = 1;
        if strcmp(blockType, 'Scope') == 1
            getProps = 0;
        elseif numOfParams < 1
            getProps = 0;
        end
        
        % Get block parameters:
        % ======================
        if getProps > 0
            for y = 1:numOfParams                
                paramName = ObjParams{y}.Prompt;                
                paramVal = get_param(internalBlockPath, InternalNames{y});
                if ischar(paramName) && ischar(paramVal)
                    modelParser.addProperty(paramName, paramVal);
                end
            end
        end
        
        % get sub system data:
        % ======================        
        if strcmp(blockType, 'SubSystem') == 1
            subsys = find_system(sys, 'name', internalBlockName);
            if isobject(subsys) == 0
                openFunc = get_param(subsys, 'OpenFcn');
                if isempty(openFunc)
                    getSystemData(subsys, modelParser, sysPath, helper, publishDir);
                end
            end
        end
    end
    
    
function printSystemDiagram(sys, publishDir, helper)

    orient(sys,'landscape');

    internalSysName = get_param(sys,'name');
    tempSysUriName = strrep(internalSysName, '/', '_');
    sysUriName = char(helper.legalizeMdlElemName(tempSysUriName));
    set(sys, 'PaperOrientation' , 'portrait');
    diagPicDir = strcat(publishDir,'/', sysUriName ,'.bmp');
    saveas(sys, diagPicDir);

     
function printChartDiagram(sys, sysPath, publishDir, helper)

    orient(sys,'landscape');

    internalSysName = get_param(sys,'name');
    tempSysUriName = strrep(internalSysName, '/', '_');
    sysUriName = char(helper.legalizeMdlElemName(tempSysUriName));
    set(sys, 'PaperOrientation' , 'portrait');
    diagPicDir = strcat(publishDir,'/', sysUriName ,'.bmp');
    sfprint(sysPath,'bitmap',diagPicDir);
